#!/usr/bin/python
# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Main init script. Loads plugins and resolves their dependencies."""

import gtk
import os
import gobject
import sys

from src import gui
from src import playlistmanager
from src import config
from src import util

def load_backends(conf):
  names = [x for x in (x.strip().lower() for x in
                       conf.get("general", "backends").split(","))
           if x != ""]
  to_load = names[:]
  modules = []

  while len(to_load) > 0:
    name = to_load.pop(0)
    full_name = "src.backends."+name
    if not full_name in sys.modules:
      print "Loaded",full_name
      __import__(full_name)
    mod = sys.modules[full_name]
    if all(pair[0] != name for pair in modules):
      modules.append((name, mod))
    to_load.extend(filter(lambda x: all(pair[0] != x for pair in modules),
                          mod.DEPS))

  plugins = {}

  while len(modules) > 0:
    name, mod = modules.pop(0)
    if all(x in plugins for x in mod.DEPS):
      plugins[name] = mod.Plugin(plugins, conf)
    else:
      modules.append((name, mod))

  result = [plugins[name] for name in names]

  return result

def load_frontends(playlist, conf):
  names = [x for x in (x.strip().lower() for x in
                       conf.get("general", "frontends").split(","))
           if x != ""]

  plugins = {}

  for name in names:
    full_name = "src.frontends."+name
    if not full_name in sys.modules:
      print "Loaded", full_name
      __import__(full_name)
    mod = sys.modules[full_name]
    plugins[name] = mod.Plugin(playlist, conf)
  return plugins

def main():
  print "Initializing..."
  # disable X11 display power management and screensaver
  try:
    os.system("xset -dpms s off")
  except:
    pass

  conf = config.get_config()

  util.prepare_cache(conf.get("general", "cache_dir"))

  gobject.threads_init()

  backends = load_backends(conf)

  m = playlistmanager.PlaylistManager(backends, conf)

  frontends = load_frontends(m, conf)

  g = gui.Gui(m, conf)

  print "Done loading modules..."
  print "Starting modules..."

  for thread in frontends.values():
    thread.start()

  def exit(*args):
    print "Shutting down..."
    for thread in frontends.values():
      thread.kill()
    g.destroy()
    try:
      gtk.main_quit()
    except RuntimeError:
      pass
    print "Done shutting down."

  g.connect("delete-event",exit)

  g.show()

  print "Done initializing."
  try :
    gtk.main()
  except KeyboardInterrupt:
    exit()

if __name__ == "__main__":
  main()

