# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Fullscreen gtk interface."""

import gtk
import gobject
import pango
import os

from . import config
from . import util

class Gui(gtk.Window):
  def __init__(self, player, conf):
    gtk.Window.__init__(self)
    gtk.rc_parse(util.get_resource_path("gtkrc"))
    self.set_title(config.app_name)
    self.config = conf

    self.connect("expose_event", self._expose)

    self.player = player
    self.player.connect("stopped", self._stopped)
    self.player.connect("metadata_updated", self._metadata_updated)
    self.player.connect("item_added", self._item_added)
    self.player.connect("item_removed", self._item_removed)
    self.player.connect("playing", self._new_playing)
    self.player.connect("items_swapped", self._items_swapped)

    self.vbox = gtk.VBox()
    self.add(self.vbox)

    self.video = gtk.VBox()
    self.vbox.pack_start(self.video, True, True)

    self.notvideo = gtk.VBox()
    self.vbox.pack_start(self.notvideo, True, True)

    self.playlist = gtk.ListStore(gobject.TYPE_ULONG, # ID
                                  gobject.TYPE_STRING, # Title
                                  gobject.TYPE_STRING, # Artist
                                  gobject.TYPE_STRING, # Album
                                  gobject.TYPE_STRING, # Time
                                  gobject.TYPE_STRING) # Pathname
    pview = gtk.TreeView(self.playlist)
    pview.set_property("rules-hint", True)
    pane = gtk.ScrolledWindow()
    pane.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
    pane.add(pview)
    self.notvideo.pack_start(pane, True, True)
    
    name_col = gtk.TreeViewColumn("Title")
    name_cell = gtk.CellRendererText()
    name_col.pack_start(name_cell, True)
    name_col.add_attribute(name_cell, 'text', 1)
    name_col.set_expand(True)
    pview.append_column(name_col)

    artist_col = gtk.TreeViewColumn("Artist")
    artist_cell = gtk.CellRendererText()
    artist_col.pack_start(artist_cell, True)
    artist_col.add_attribute(artist_cell, 'text', 2)
    artist_col.set_expand(True)
    pview.append_column(artist_col)

    album_col = gtk.TreeViewColumn("Album")
    album_cell = gtk.CellRendererText()
    album_col.pack_start(album_cell, True)
    album_col.add_attribute(album_cell, 'text', 3)
    album_col.set_expand(True)
    pview.append_column(album_col)

    time_col = gtk.TreeViewColumn("Duration")
    time_cell = gtk.CellRendererText()
    time_col.pack_start(time_cell, True)
    time_col.add_attribute(time_cell, 'text', 4)
    time_col.set_expand(False)
    pview.append_column(time_col)

    for cell in (name_cell, artist_cell, album_cell, time_cell):
      cell.set_property("ellipsize", pango.ELLIPSIZE_END);
      cell.set_property("ellipsize-set", True)

    self.log_widget = gtk.Label("Initializing...")
    self.log_widget.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
    self.notvideo.pack_start(self.log_widget, False, False)

    self.vbox.show()
    self.notvideo.show_all()

    self.set_video(False)

    #hacky fullscreen
    self.set_default_size(self.get_screen().get_width(),
                          self.get_screen().get_height())
    self.maximize()
    self.fullscreen()

    #hacky pointer move
    self.get_display().warp_pointer(self.get_screen(),
                                    self.get_screen().get_width()-1,
                                    self.get_screen().get_height()-1)

  def _get_iter_for_row(self, media):
    for i in range(len(self.playlist)):
      if self.playlist[i][0] == media.id:
        return self.playlist.get_iter(i)
    return None
  def _get_entry_for_row(self, media):
    entry = (media.id,
             media.metadata["title"] if "title" in media.metadata and
               media.metadata["title"] else "N/A",
             media.metadata["artist"] if "artist" in media.metadata and
               media.metadata["artist"] else "N/A",
             media.metadata["album"] if "album" in media.metadata and
               media.metadata["album"] else "N/A",
             util.format_time(media.metadata["duration"]) if 
               "duration" in media.metadata and
               media.metadata["duration"] else "N/A",
             media.uri)
    return entry
  def _expose(self, obj, event):
    pixmap = gtk.gdk.Pixmap(None, 1, 1, 1)
    color = gtk.gdk.Color()
    cursor = gtk.gdk.Cursor(pixmap, pixmap, color, color, 0, 0)
    self.window.set_cursor(cursor)
  def _stopped(self, object):
    self.set_video(False)
    self.log("Stopped...")
  def _metadata_updated(self, object, media):
    self.log("Metadata updated for " + media.uri)
    iter = self._get_iter_for_row(media)
    if iter is not None:
      if (self.playlist.get_path(iter) ==
          self.playlist.get_path(self.playlist.get_iter_first())):
        self._process_video(media)
      self.playlist.insert_after(iter, self._get_entry_for_row(media))
      self.playlist.remove(iter)
  def _item_added(self, object, media):
    self.log("Item added " + media.uri)
    self.playlist.append(self._get_entry_for_row(media))
  def _item_removed(self, object, media):
    self.log("Item removed " + media.uri)
    iter = self._get_iter_for_row(media)
    if iter is not None:
      self.playlist.remove(iter)
  def _items_swapped(self, object, row1, row2):
    iter1 = self._get_iter_for_row(row1)
    iter2 = self._get_iter_for_row(row2)
    if iter1 is not None and iter2 is not None:
      self.playlist.swap(iter1, iter2)
  def _new_playing(self, player, object):
    self.log("Now playing " + object.uri)
    self._process_video(object)
  def _process_video(self, object):
    children = self.video.get_children()
    for child in children:
      child.hide()
    if object.is_video():
      child = object.get_widget()
      if not (child in children):
        self.video.pack_start(child, True, True)
      child.show()
      self.set_video(True)
  def set_video(self, video):
    if (video):
      self.notvideo.hide()
      self.video.show()
    else:
      self.notvideo.show()
      self.video.hide()
  def log(self, text):
    self.log_widget.set_text(text)
