# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Manages the playlist."""

import threading
import gobject

from . import util
from . import playlistitem

class PlaylistManager(gobject.GObject):
  __gsignals__ = {
    "playing" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                     (gobject.TYPE_PYOBJECT,)),
    "stopped" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
    "item_added" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                    (gobject.TYPE_PYOBJECT,)),
    "item_removed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                      (gobject.TYPE_PYOBJECT,)),
    "metadata_updated" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                          (gobject.TYPE_PYOBJECT,)),
    "items_swapped" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                       (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT))
  }
  def __init__(self, backends, conf):
    gobject.GObject.__init__(self)

    self.config = conf
    self.backends = backends
    self.queue = []
    self.bind = None
    self.lock = threading.RLock()
    self.last_id = 0
    self.volume = self.config.getint("general", "startup_volume")

  def add_item(self, uri, temp, immediate=False):
    backend = self.get_backend_for_uri(uri)
    if backend is not None:
      with self.lock:
        self.last_id += 1
        item = playlistitem.PlaylistItem(self.last_id, uri, backend, temp)
        util.idle_add(item.connect, "metadata_loaded", self._relay_metadata)
        self.queue.append(item)
        util.idle_add(self.emit, "item_added", item)
        if len(self.queue) == 1:
          self.play_next()
        elif immediate:
          for i in range(len(self.queue) - 2):
            self.move_up(item.id)
          temp = self.queue[0]
          self.queue.append(temp)
          util.idle_add(self.emit, "item_added", temp)
          self.remove(temp.id)
          for i in range(len(self.queue) - 2):
            self.move_up(temp.id)
  def _relay_metadata(self, item):
    self.emit("metadata_updated", item)
  def _stopped(self, item):
    self.emit("stopped")
    with self.lock:
      if len(self.queue) > 0 and self.queue[0] is item:
        removed = self.queue.pop(0)
        if not removed in self.queue:
          removed.unbind()
        if self.bind is not None:
          self.bind[0].disconnect(self.bind[1])
          self.bind = None
        self.emit("item_removed", removed)
      self.play_next()
  def play_next(self):
    with self.lock:
      if (len(self.queue) > 0):
        util.idle_add(self._play)
  def _play(self):
    self.queue[0].backend.set_volume(self.volume)
    self.queue[0].play()
    self.bind = (self.queue[0],
                 self.queue[0].connect("stopped", self._stopped))
    self.emit("playing", self.queue[0])
  def get_backend_for_uri(self, uri):
    for backend in self.backends:
      if backend.can_play(uri):
        return backend
    return None
  def remove(self, id):
    with self.lock:
      i = self._get_index_for_id(id)
      if i is not None:
        if i > 0:
          removed = self.queue.pop(i)
          removed.unbind()
          util.idle_add(self.emit, "item_removed", removed)
        elif i == 0:
          self.queue[0].stop()
  def move_down(self, id):
    with self.lock:
      i = self._get_index_for_id(id)
      if i is not None and i < len(self.queue) - 1:
        self.queue[i], self.queue[i+1] = self.queue[i+1], self.queue[i]
        util.idle_add(self.emit, "items_swapped",
                      self.queue[i], self.queue[i+1])
  def move_up(self, id):
    with self.lock:
      i = self._get_index_for_id(id)
      if i is not None and i > 1:
        self.queue[i], self.queue[i-1] = self.queue[i-1], self.queue[i]
        util.idle_add(self.emit, "items_swapped",
                      self.queue[i], self.queue[i-1])
  def _get_index_for_id(self, id):
    with self.lock:
      for i in range(len(self.queue)):
        if self.queue[i].id == id: 
          return i
      return -1
  def modify_current_volume(self, val):
    with self.lock:
      self.volume += val
      self.volume = 0 if self.volume < 0 else (100
                        if self.volume > 100 else self.volume)
      if len(self.queue) > 0:
        self.queue[0].backend.set_volume(self.volume)
  def get_current_volume(self):
    return self.volume
  def get_queue(self): 
    with self.lock:
      return self.queue[:]
